RichError

Struct RichError 

pub struct RichError {
    pub code: ErrorCode,
    pub message: String,
    pub broad_span: Span,
    pub label: Option<Label>,
    pub notes: Vec<Note>,
}
A highly detailed error type designed for:

  • Compilers (or anything similar)
  • Tracebacks
  • Any situation where a source file would provide valuable context for the user

This serves as a sort of wrapper around ariadne, although it is missing much of ariadne’s functionality. The point is to have a simple interface for constructing error reports that works well enough for sufficiently simple applications.

As a word of caution, this higher level of detail comes at the cost of higher memory usage. The struct alone is around 7 times larger than a String, and some features may incur extra heap allocations as well. Thus, it is not advisable to use rich errors unless an ordinary error is truly insufficient.

Fields

code: ErrorCodemessage: String

The error message.

This is meant to be a description of the general error specified by code rather than a verbose description of precisely what failed. More details should be sent through label and/or notes.

broad_span: Span

The broad area in the source file that caused the error.

The exact location of the error can be specified via label.

label: Option<Label>

An additional error message associated with a specific area of the source code.

notes: Vec<Note>

An arbitrary number of notes that provide additional context and/or assistance to the user.

Implementations

Source

impl RichError

pub fn new<E>(err: E, broad_span: Span) -> Self
where E: ToErrorCode + ToString,

Constructs a new rich error.

To add additional metadata to this message, use the relevant builder methods.

pub fn with_note<S>(self, message: S) -> Self
where S: ToString,

Adds an ordinary note to this error.

pub fn with_help<S>(self, message: S) -> Self
where S: ToString,

Adds a note with a help message to this error.

pub fn with_label(self, label: Label) -> Self

Adds a label to this error.

If a label was already present, this overwrites it.

pub fn with_narrow_span(self, span: Span) -> Self

Specifies the exact location of the error within the broader span.

This is similar to creating a blank label with the provided span, but it also handles the case where label was already set. In that case, label’s message will be preserved while its span is overwritten.

pub fn report<P>(self, source: &str, source_path: P) -> Result<()>
where P: AsRef<Path>,

Reports the error to the user.

This is a relatively expensive operation, so depending on the use case, it might be wise to either report all errors at once or have a separate thread report errors as they crop up.